﻿// -*- Encoding: UTF-8 -*-
#include "FreeRTOSCommon.h"
#include "Accelerometer.h"
#include "I2CDriver.h"
#include "mymath.h"

#define ACCEL_SENSOR_DEVICE_ADDR   0x1D
#define ACCEL_SENSOR_OUT_X_REG     0x01
#define ACCEL_SENSOR_CTRL_REG1     0x2A
#define ACCEL_SENSOR_CTRL_REG2     0x2B

static int JoinByteToInt(const uint8_t* bytes)
{
  uint32_t value = ((uint32_t)bytes[0] << 2) | (bytes[1] >> 6);
  if (value & 0x200) {
    value = ((~value) + 1) & 0x3FF;
    return -(int)value;
  }
  return (int)value;
}

static int Calc3AxisResultant(const uint8_t* data)
{
  int x, y, z;
  
  x = JoinByteToInt(data);
  y = JoinByteToInt(data + 2);
  z = JoinByteToInt(data + 4);
  
  return IntSqrt(x*x + y*y + z*z);
}

static bool GetSensorValue(uint8_t *data)
{
  return I2CRegRead(ACCEL_SENSOR_DEVICE_ADDR, ACCEL_SENSOR_OUT_X_REG, data, 6);
}

/**
  xyzの3軸合成値を返却する
  (負数は取得失敗)
**/
int GetCurrent3AxisResultant(void)
{
  uint8_t data[6];
  if (GetSensorValue(data)) {
    return Calc3AxisResultant(data);
  }
  return -1;
}

void InitializeAcceleroMeter(void)
{
  InitializeI2C();
  vTaskDelay(100);
  
  // Oversamplingの設定をHigh Resolutionにする
  // CTRL_REG2.MODS[1:0](Bit 1:0) = 2'b10
  I2CRegWriteByte(ACCEL_SENSOR_DEVICE_ADDR, ACCEL_SENSOR_CTRL_REG2, 0x02);
  
  // データレートは 12.5Hz(80ms周期) に設定
  // CTRL_REG1.DR[2:0](Bit 5:3) = 5
  // CTRL_REG1.ACTIVE (Bit 0:0) = 1
  I2CRegWriteByte(ACCEL_SENSOR_DEVICE_ADDR, ACCEL_SENSOR_CTRL_REG1, (0x05 << 3) | 0x01);
}

