﻿// -*- Encoding: UTF-8 -*-
#include "FreeRTOSCommon.h"
#include "PitchAdjust.h"
#include "GPS.h"
#include "PitchMeasure.h"
#include "DestinationMgr.h"
#include <stdlib.h>

#define STEP_INTERVAL_ADJUSTMENT_STEP     12    // 12ms => 5 BPMに相当
#define STEP_INTERVAL_MIN                 290   // 290msec => 約207 BPM相当

void UpdateGPSStepInfo(GPS_STEP_INFO *info)
{
  info->valid = TICK_INFO_VALID;
  info->tick = xTaskGetTickCount();
  if (IsWalking()) {
    info->valid |= STEP_INFO_VALID;
    info->stepCount = GetStepCounter();
  }
  if (IsGPSActive()) {
    info->valid |= TIMESTAMP_INFO_VALID | LAT_LNG_INFO_VALID;
    info->timestamp = GetGPSTimestamp();
    GetGPSLatLng(&(info->lat), &(info->lng));
  }
}

bool IsNeededInfoValid(GPS_STEP_INFO *info)
{
  if ((info->valid & NEEDED_INFO_MASK) == NEEDED_INFO_MASK) {
    return true;
  }
  return false;
}

int CalcStepInterval(GPS_STEP_INFO *prev, GPS_STEP_INFO *new)
{
  uint32_t timeDiff = new->tick - prev->tick;
  uint32_t stepDiff = new->stepCount - prev->stepCount;
  if (stepDiff == 0) {
    return timeDiff;
  }
  return (timeDiff / stepDiff);
}

int CalcPreferedStepInterval(int currentStepInterval,
                             GPS_STEP_INFO *prev, GPS_STEP_INFO *new,
                             int *diffArrivalTime)
{
  uint32_t diffTick;
  double diffDistance;
  double remainingDistance;
  int remainingSeconds;
  int estimatedArrivalTime;
  int scheduledArrivalTime;
  int diffArrivalTimeAbs;
  int correctionValue;
    
  diffTick = new->tick - prev->tick;
  diffDistance = CalcGPSDistance(prev->lat, prev->lng,
                                 new->lat, new->lng);
  if (diffDistance < 0.002) {
    *diffArrivalTime = 0;
    return currentStepInterval;
  }
  
  // 残距離
  remainingDistance = GetCurrentToNextDestinationDistance(new->lat, new->lng);
  remainingDistance += GetNextToLastDestinationDistance();
  
  // 残時間(秒)
  remainingSeconds = (remainingDistance * diffTick) / (diffDistance * 1000);
  
  // 到着予想時間, 目標時刻(秒)
  estimatedArrivalTime = new->timestamp + remainingSeconds;
  scheduledArrivalTime = GetFinalScheduledArrivalTime();
  
  // 到着時刻とのズレの割合により歩数を調整する
  *diffArrivalTime = estimatedArrivalTime - scheduledArrivalTime;
  diffArrivalTimeAbs = abs(*diffArrivalTime);

  if (diffArrivalTimeAbs < 60) {
    correctionValue = 0;
  } else {
    correctionValue = ((diffArrivalTimeAbs / 30) - 1) * STEP_INTERVAL_ADJUSTMENT_STEP;
  }
  if (*diffArrivalTime > 0) {
    correctionValue = -correctionValue;
  }
  if ((currentStepInterval + correctionValue) < STEP_INTERVAL_MIN) {
    return STEP_INTERVAL_MIN;
  }
  return currentStepInterval + correctionValue;
}
